// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Proxy.h	--- container class for reference counted objects
//
//	DESCRIPTION:
//		Proxy is a simpe containter class that implementes reference
//		counting for objects.
//
//	CHANGE HISTORY:
//		UL, 2000-03-08: 1.0.0: - first version
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/caloalutils/src/Proxy.h,v 1.3 2004/08/04 12:18:14 rsaxton Exp $
//

#ifndef PROXY_H
#define PROXY_H

template<class PointedType>
class Proxy {
public:
	//
	//	default construction
	//
	Proxy() : _ptr(0), _pcount(0) {}

	//
	//	construction from pointer to PointedType
	//
	Proxy(PointedType *p) : _ptr(p), _pcount(new unsigned(1)) {}

	//
	//	copy construction
	//
	Proxy(const Proxy &rhs) : _ptr(rhs._ptr), _pcount(rhs._pcount) {
		(*_pcount)++;
	}

	//
	//	assignment
	//
	Proxy &operator=(const Proxy &rhs) {
		if (_ptr == rhs._ptr)
			return *this;

		//	check if this is the last reference to object - if yes, delete
		//	it + reference counter
		if (_pcount && *_pcount <= 1) {
			delete _ptr;
			delete _pcount;
		}

		_ptr = rhs._ptr;
		_pcount = rhs._pcount;
		(*_pcount)++;

		return *this;
	}


	//
	//	de-referencing
	//
	PointedType *operator->() const { return _ptr; }
	PointedType &operator*() const { return *_ptr; }

	//
	//	destruction
	//
	~Proxy() {
		if (_pcount)
			if (--(*_pcount) == 0) {
				//	this is the last reference - delete object + associated
				//	pointer
				delete _ptr;
				delete _pcount;
			}
	}

private:
	PointedType	*_ptr;		//	pointer to object
	unsigned	*_pcount;	//	pointer to associated object counter
};

#endif
