// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	RealCcf.h	--- interface between the CAL and the CCF
//
//	DESCRIPTION:
//		This defines the interface between the CCF and the CAL
//
//	CHANGE HISTORY:
//		UL, 1997-10-10: 1.0.0: - skeleton
//		UL, 2000-10-05: 1.1.0: - added RealCcf()::cif() to allow
//								 befriended classes to get access to the CIF
//
//	AUTHOR:
//		U. Lammers, Fri Oct 10 12:43:51 WET DST 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//		[2]: Indexing of the XMM Current Calibration File, U. Lammers,
//			 15/07/97
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/caloalutils/src/RealCcf.h,v 1.4 2004/08/04 12:18:14 rsaxton Exp $
//

#ifndef REAL_CCF_H
#define REAL_CCF_H

#include "Ccf.h"

class RealCcf : public virtual Ccf {
public:
	RealCcf(const std::string cifFileName = "");
		// open a CIF defining the CCF

	//
	//	register new CifEntry
	//
	void registerNewCifEntry(const std::string &ccfSetName);

	//	query for a calibration file of a particular type corresponding to an
	//	instrument
	bool hasEntry(const XmmInstrument *, Ccf::CalDataTypes) const;
	CifEntry *entry(const XmmInstrument *, Ccf::CalDataTypes,
					bool generateError = true) const;

	//	print
	void print(std::ostream &os) const;

	~RealCcf();

private:
	Cif _cif;
									// the CIF: a map with pair of
									// instrument/datatype and corresponding
									// pointers to CIF entries

	friend class CalviewGui;
	friend class ShowCcf;
	const Cif &cif() const { return _cif; }
		//
		//	PURPOSE:
		//		Give friends access to CIF
		//

	void ccfReplacements();
};

#endif
