// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	odfdesignators_c.cc --- implementation of C I/F layer to OdfDesignator
//
//	DESCRIPTION:
//		See odfdesignators.f90
//
//	CHANGE HISTORY:
//		UL, 2002-01-08: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers,	Tue Jan  8 17:28:19 MET 2002
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/caloalutils/src/odfdesignators_c.cc,v 1.2 2004/08/04 12:18:14 rsaxton Exp $
//

#include "common.h"
#include "f90cxx.h"
#include "Handle.h"
#include "OdfDesignators.h"
#include "CalOalUtility.h"

using namespace CalOalUtility;

#define CSTC(ptr)   ((const InstrumentExposureIdentifier *)ptr)

typedef Handle<InstrumentExposureIdentifier> InstExpIdHandle;

extern "C" {

InstExpIdHandle CXXFUNC(instrumentexposureidentifier,
						instrumentExposureIdentifier)(const char *designator,
													  Integer3 len)
{
	static InstrumentExposureIdentifier insExpId("M1S001");
	insExpId = InstrumentExposureIdentifier(f90string(designator, len));

	return &insExpId;
} // instrumentExposureIdentifier()

Integer3 CXXFUNC(instrument, instrument)(const InstExpIdHandle *insExpIdH)
{
	return Integer3(insExpIdH->addressOfObject()->instrument());
} // instrument()

Logical CXXFUNC(scheduleflagapplicable, scheduleFlagApplicable)
											(const InstExpIdHandle *insExpIdH)
{
	return Logical(insExpIdH->addressOfObject()->scheduleFlagApplicable());
} // scheduleFlagApplicable()

Logical CXXFUNC(scheduled, scheduled)(const InstExpIdHandle *insExpIdH)
{
	return Logical(insExpIdH->addressOfObject()->scheduled());
} // scheduled()

Logical CXXFUNC(unscheduled, unscheduled)(const InstExpIdHandle *insExpIdH)
{
	return Logical(insExpIdH->addressOfObject()->unscheduled());
} // unscheduled()

Integer3 CXXFUNC(exposurenumber, exposureNumber)
											(const InstExpIdHandle *insExpIdH)
{
	return Integer3(insExpIdH->addressOfObject()->exposureNumber());
} // exposureNumber()

void CXXFUNC(exposureid, exposureId)(char *str, Integer3 len,
									 const InstExpIdHandle *insExpIdH)
{
	stringf90(insExpIdH->addressOfObject()->exposureId(), str, len);
} // exposureId()

void CXXFUNC(instrumentexposureid, instrumentExposureId)
					(char *str, Integer3 len, const InstExpIdHandle *insExpIdH)
{
	stringf90(insExpIdH->addressOfObject()->instrumentExposureId(), str, len);
} // exposureId()

}
